#include <stdio.h>
#include <iostream.h>
#include <config.h>

#include "generic.h"
#include "gui_muse.h"
#include "gui_pixmaps.h"
#include "jmixer.h"
#include "libsync/synchronous.h"
#include <FL/fl_ask.H>
#include <FL/fl_file_chooser.H>

static LTI::Synchronous *dealer;
static Stream_mixer *mixer;


/* where to store volumes for the talk func */
static float store_vol[MAX_CHANNELS];

/* resolves the bitrate widget choice */
int solve_bps(int bps_choice) {
  int bps = 0;

  switch(bps_choice) {
  case 0:
    bps = 8;
    break;
  case 1:
    bps = 16;
    break;
  case 2:
    bps = 24;
    break;
  case 3:
    bps = 32;
    break;
  case 4:
    bps = 56;
    break;
  case 5:
    bps = 64;
    break;
  case 6:
    bps = 96;
    break;
  case 7:
    bps = 128;
    break;
  }
  return(bps);
}

int solve_frequency(int freq) {
  int f;
  
  switch(freq) {
  case 0:
    f = freq;
    break;
  case 1:
    f = 11000;
    break;
  case 2:
    f = 16000;
    break;
  case 3:
    f = 22050;
    break;
  case 4:
    f = 44100;
    break;
  }

  return(f);
}

void init_callbacks(Stream_mixer *mix, const LTI::Synchronous *deal) {
  mixer = mix;
  dealer = (LTI::Synchronous *)deal;
}

inline void sync_mixer() {
  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();
}

void set_channel(Fl_Widget *o, void *ch, int song){

  sync_mixer();

  /* playlist starts from 0   Fl browser value starts from 1 */ 

   if(mixer->set_channel(*(int*)ch,(song-1))) {
     ((GuiChannel *)(o->parent()->user_data()))->playlist_browser->value(song);
     ((GuiChannel*)(o->parent()->user_data()))->timeline_slider->value(0.0);
     ((GuiChannel*)(o->parent()->user_data()))->time->value("00:00:00");
     ((GuiChannel*)(o->parent()->user_data()))->name->value(((GuiChannel*)(o->parent()->user_data()))->playlist_browser->text(song));
     
   }
}

void cb_set_channel(Fl_Widget *o, void *ch) {
  int song = ((Fl_Select_Browser *)o)->value();
  set_channel(o, ch, song);
}

void cb_set_channel_plus(Fl_Widget *o, void *ch) {  
   int song = ((GuiChannel *)(o->parent()->user_data()))->playlist_browser->value();
   song++;
   if (song <= ((GuiChannel *)(o->parent()->user_data()))->playlist_browser->size())
     set_channel(o, ch, song);
   else cout << "no more left \n";
}

void cb_set_channel_minus(Fl_Widget *o, void *ch) {  
   int song = ((GuiChannel *)(o->parent()->user_data()))->playlist_browser->value();
   song--;
   if (song > 0)
     set_channel(o, ch, song);
   else cout << "no more in playlist \n";
}

void cb_play_channel(Fl_Widget *o, void *ch) {

  sync_mixer();
  mixer->play_channel(*(int*)ch);
  
}

void cb_stop_channel(Fl_Widget *o, void *ch) {
  sync_mixer();
  if(!mixer->stop_channel(*(int*)ch)) {
    char temp[100];
    sprintf(temp,"Something weird happened trying to stop channel [%u]",*(int*)ch);
    fl_alert(temp);
  } else {
    ((GuiChannel*)(o->parent()->user_data()))->timeline_slider->value(0.0);
    ((GuiChannel*)(o->parent()->user_data()))->time->value("00:00:00");
  }
}

void cb_pause_channel(Fl_Widget *o, void *ch) {
  sync_mixer();
  mixer->pause_channel(*(int*)ch);
}

void cb_set_volume(Fl_Widget *o, void *ch) {
  float vol = ((Fl_Slider *)o)->value();
  sync_mixer();
  mixer->set_volume(*(int*)ch,vol);
}

void cb_set_speed(Fl_Widget *o, void *ch) {
  int speed = (int) ((Fl_Slider *)o)->value();
  sync_mixer();
  mixer->set_speed(*(int*)ch,speed);
}

void cb_set_position(Fl_Widget *o, void *ch) {
  float pos = ((Fl_Slider *)o)->value();
  sync_mixer();
  if(!mixer->set_position(pos,*(int*)ch)) {
    char temp[100];
    sprintf(temp,"Urg! it seems i can't seek position on channel [%u]",*(int*)ch);
    fl_alert(temp);
  }
}

void cb_set_playmode(Fl_Widget *o, void *ch) {
  int i = *(int*)ch;

  switch (mixer->xach[i]->playmode)
    {
    case 0: /* LOOP -> PLAY */
      o->label("play");
      mixer->xach[i]->playmode = mixer->xach[i]->PLAY;
      break;
    case 1: /* CONT -> LOOP */
      o->label("loop");
      mixer->xach[i]->playmode = mixer->xach[i]->LOOP;
      break;
    case 2: /* PLAY -> CONT */
      o->label("cont");
      mixer->xach[i]->playmode = mixer->xach[i]->CONT;
      break;
    }

}

void cb_set_icecast(Fl_Widget *o) {
  Amuse *tgui;
  tgui = (Amuse*)(o->parent()->parent()->parent()->user_data());
  
  if(((Fl_Light_Button *)o)->value()==0) { /* stop the stream */
    
    sync_mixer();
    mixer->stop_icecast();
    
    tgui->ip_input->activate();
    tgui->port_input->activate();
    tgui->mount_input->activate();
    tgui->name_input->activate();
    tgui->url_input->activate();
    tgui->description_input->activate();
    tgui->pass_input->activate();
    tgui->connect_button->label("connect");
    tgui->bitrate_choice->activate();
    tgui->login_choice->activate();

  } else { /* set the stream */
    
    char *ip, *port, *mount, *pass, *name, *url, *desc;
    int bps, login;
    int frequency, lowpass, highpass, mode, quality;
    unsigned long int nport;

    ip = (char*)tgui->ip_input->value();
    port = (char*)tgui->port_input->value();
    nport = strtol(port,(char **)NULL,10);
    
    mount = (char*)tgui->mount_input->value();
    pass = (char*)tgui->pass_input->value();
    bps = tgui->bitrate_choice->value();

    name = (char*)tgui->name_input->value();
    url = (char*)tgui->url_input->value();
    desc = (char*)tgui->description_input->value();

    frequency = tgui->resample_choice->value();
    mode = tgui->channels_choice->value();
    quality = (int)tgui->quality_slide->value();
    
    switch(tgui->filtering_choice->value()) {
    case 0:
      lowpass = 0; highpass = 0;
      break;
    case 1:
      lowpass = -1; highpass = -1;
      break;
    case 2:
      lowpass = atoi(tgui->lowpassfreq_input->value());
      highpass = atoi(tgui->highpassfreq_input->value());
      break;
    }

    bps = solve_bps(bps);
    frequency = solve_frequency(frequency);
    login = tgui->login_choice->value();
    
    sync_mixer();

    mixer->set_lame(bps, frequency, mode, quality, lowpass, highpass);

    sync_mixer();

    if(mixer->set_icecast(ip,nport,pass,mount,bps, name, url, desc, login))
      ((Fl_Light_Button *)o)->set();
    else
      ((Fl_Light_Button *)o)->clear();
  }

}

void cb_set_talk(Fl_Widget *o) {
  Amuse *tgui;
  int i;
  float temp_vol[MAX_CHANNELS];

  tgui = (Amuse *)(o->parent()->user_data());

  if(((Fl_Light_Button *)o)->value()==1) { 
    /* talk is not active; activate it */  
    for (i = 0; i < MAX_CHANNELS; i++) {
      store_vol[i] = tgui->chan[i]->volume_slider->value();
      if(store_vol[i]<TALK_VOLUME)
	temp_vol[i] = 0.0;
      else
	temp_vol[i] = TALK_VOLUME;
      tgui->chan[i]->volume_slider->value(temp_vol[i]);
    }

    sync_mixer();
    mixer->set_all_volumes(&temp_vol[0]);

  } else {   
    /* deactivate talk : bring volumes back up */
    for (i = 0; i < MAX_CHANNELS; i++)
      tgui->chan[i]->volume_slider->value(store_vol[i]);
    
    sync_mixer();
    mixer->set_all_volumes(&store_vol[0]);
    
  }

}

void cb_set_linein(Fl_Widget *o) {
  bool res;

  sync_mixer();

  res = mixer->set_live(((Fl_Light_Button *)o)->value());

  if(res)
    ((Fl_Light_Button *)o)->set();
  else
    ((Fl_Light_Button *)o)->clear();
}

void cb_set_lineout(Fl_Widget *o) {
  bool res;

  sync_mixer();

  res = mixer->set_lineout(((Fl_Light_Button *)o)->value());
  
  if(res)
    ((Fl_Light_Button *)o)->set();
  else
    ((Fl_Light_Button *)o)->clear();
  
}

void cb_save_file(Fl_Widget *o) {
  const char *filename;
  Amuse *tgui;

  tgui = (Amuse*)(o->parent()->parent()->parent()->user_data());
  filename = tgui->filesave_input->value();

  /* TURNING ON */
  if (((Fl_Light_Button *)o)->value() == 1)
    {
      if ((filename[0] == '\0') || (filename[0] ==  ' '))   
	{
	  tgui->status_main->value("you need to type in a filename");
	  tgui->filesave_button->value(0);
	} else {
	  int res, bps, freq, mode, quality, lowpass, highpass;	  

	  /* retrieve lame settings */
	  bps = solve_bps(tgui->bitrate_choice->value());
	  freq = tgui->resample_choice->value();
	  mode = tgui->channels_choice->value();
	  quality = (int) tgui->quality_slide->value();
	  
	  switch(tgui->filtering_choice->value()) {
	  case 0:
	    lowpass = 0; highpass = 0;
	    break;
	  case 1:
	    lowpass = -1; highpass = -1;
	    break;
	  case 2:
	    lowpass = atoi(tgui->lowpassfreq_input->value());
	    highpass = atoi(tgui->highpassfreq_input->value());
	    break;
	  }
	  freq = solve_frequency(freq);
	  
	  sync_mixer();
	  mixer->set_lame(bps, freq, mode, quality, lowpass, highpass);

	  /* input syntax ok, let's start recording */
	  sync_mixer();
	  res = mixer->set_mp3file((char*)filename, bps);
	  /* if res -1 should notice error */

	  tgui->filebrowse_button->deactivate();
	  tgui->filesave_input->deactivate();
	  tgui->status_main->value(filename);
	}
    }

  /* TURNING OFF */
  if (((Fl_Light_Button *)o)->value() == 0)
    {
      /* let's shutdown file recording */
      sync_mixer();
      mixer->stop_mp3file();

      tgui->filebrowse_button->activate();
      tgui->filesave_input->activate();
      tgui->status_main->value(" ");
      
    }
}

void cb_browse_file(Fl_Widget *o) {
    char *filename;
    Amuse *tgui;
    tgui = (Amuse*)(o->parent()->parent()->parent()->user_data());
    filename = fl_file_chooser("Pick a file", "*", "");
    tgui->filesave_input->value(filename); 
}

void cb_browse_playlist(Fl_Widget *o, void *ch) {
  char *filename;
  int i = *(int *)ch;
  Amuse *tgui;
  tgui = (Amuse*)(o->parent()->parent()->parent()->parent()->user_data());
  filename = fl_file_chooser("Pick a playlist", "*", "");

  sync_mixer();
  mixer->set_playlist(filename,i);
  //    tgui->status_main->value("ehy! where is that playlist file you said??");
}

void cb_filtering_choice(Fl_Widget *o) {
  Amuse *tgui;
  tgui = (Amuse*)(o->parent()->parent()->parent()->user_data());
  int i = tgui->filtering_choice->value();

  switch(i) {
  case 0:
  case 1:
    tgui->lowpassfreq_input->deactivate();
    tgui->highpassfreq_input->deactivate();
    break;
  case 2:
    tgui->lowpassfreq_input->activate();
    tgui->highpassfreq_input->activate();
    break;
  }
}

Fl_Window *infowindow = NULL;
inline void infoz() {

  static char temp[128];

  infowindow = new Fl_Window(300, 350, "Multiple Streaming Engine");
  
  sprintf(temp,"%s v%s\nhttp://muse.dyne.org",PACKAGE,VERSION);

  Fl_Box *box = new Fl_Box(35, 15, 250, 30, temp);
  box->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  box->labelsize(12);
  box->labelfont(1);

  Fl_Box *authors = new Fl_Box(15, 70, 250, 50, "Authors:\n   engine by jaromil\n   gui by august black\n\n");
  authors->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  authors->labelsize(12);
  authors->labelfont(1);
  
  Fl_Box *contribs = new Fl_Box(15,130,250,50, "Contributors:\n   markus seidl, newmark, asbesto\n");
  contribs->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  contribs->labelsize(11);
  contribs->labelfont(1);

  Fl_Box *thanks = new Fl_Box(15,170,250,50, "Thanks to:\n   servus.at, hell voyager, loa,\n   bbk, alex+rasty+martinez");
  thanks->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  thanks->labelsize(11);
  thanks->labelfont(1);

  Fl_Box *radio = new Fl_Box(15,220,250,50, "MuSE screams on:\n   radiocybernet - http://kyuzz.org/radiocybernet\n   fundamental radio - http://funda.ment.org\n   standup radio - http://standup.dyne.org");
  radio->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  radio->labelsize(11);
  radio->labelfont(1);

  Fl_Box *disclaimer = new Fl_Box(15,280,290,50, "MuSE comes with ABSOLUTELY NO WARRANTY\n This is Free Software, and you are welcome\n to redistribute it under certain conditions\n refer to the Gnu Public License for details");
  disclaimer->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  disclaimer->labelsize(11);
  disclaimer->labelfont(1);

  /*
    
    Fl_Box *credits = new Fl_Box(160,260,260,30, "powered by SERVUS\nhttp://www.servus.at");
    credits->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
    credits->labelsize(11);
    credits->labelfont(1);
    
  */
  
  Fl_Button *but = new Fl_Button(10, 15, 20, 20);
  but->box(FL_FLAT_BOX);
  but->down_box(FL_FLAT_BOX);
  pixmap_peach.label(but);

 
  infowindow->end();
  infowindow->show();
}

void cb_open_info(Fl_Widget *o) {
  if(infowindow!=NULL) {
    if(!infowindow->shown())
      infoz();
  } else
    infoz();
}

/*
  void cb_reshape(Fl_Widget *o){
  Amuse *tgui;
  tgui = ((Amuse*)(o->parent()->user_data()));

//    tgui->mainwindow->size(505,735);
//    tgui->status_ice->position(70, 10);
//    tgui->talk_button->position(320, 5);
//    tgui->linein_button->position(370 ,5);
//    tgui->lineout_button->position(435 ,5);
//    tgui->version->position(413 ,715);
//    tgui->significalafica->position(495 ,710);
//    tgui->status_main->position(150, 715);

//    tgui->chan[0]->new_order(10,50);
//    tgui->chan[1]->new_order(10,145);
//    tgui->chan[2]->new_order(10,240);
  
tgui->chan[3]->new_order(10,335);
//    tgui->chan[4]->new_order(10,430);
//    tgui->chan[5]->new_order(10,525);

//    tgui->mix_monitor->position(485, 365);

//    tgui->tab->resize(0, 30, 505, 700);
//    tgui->chantab->resize(0,30, 505, 680);
//    tgui->chantab->resize(0,30, 505, 680);

Fl::redraw(); 
}
*/

void gui_quit(Fl_Menu_* o, void* v) {
  ((Amuse*)(o->parent()->user_data()))->quit = true;
}
