
#include "gui_muse.h"
#include "gui_callbacks.h"
#include "gui_extras.h"
#include "gui_pixmaps.h"
#include <iostream.h>
#include "config.h"

static char muse_version[16];

Fl_Menu_Item Amuse::menu_main_menu[] = {
 {"File", 0,  0, 0, 64, 4, 0, 12, 0},
 {"&About  ", 0,  (Fl_Callback*)cb_open_info, 0, 0, 5, 0, 12, 0},
 // {"&Reshape  ", 0,  (Fl_Callback*)cb_reshape, 0, 0, 5, 0, 12, 0},
 {"&Exit  ", 0x80071,  (Fl_Callback*)gui_quit, 0, 0, 5, 0, 12, 0},
 {0},
 {0}
};

Fl_Menu_Item* Amuse::submenu_file = Amuse::menu_main_menu + 0;
Fl_Menu_Item* Amuse::submenu_file_exit = Amuse::menu_main_menu + 1;

Fl_Menu_Item Amuse::menu_bitrate_choice[] = {
 {"8", 0,  0, 0, 4, 4, 0, 12, 0},
 {"16", 0,  0, 0, 4, 4, 0, 12, 0},
 {"24", 0,  0, 0, 4, 4, 0, 12, 0},
 {"32", 0,  0, 0, 4, 4, 0, 12, 0},
 {"56", 0,  0, 0, 4, 4, 0, 12, 0},
 {"64", 0,  0, 0, 4, 4, 0, 12, 0},
 {"96", 0,  0, 0, 4, 4, 0, 12, 0},
 {"128", 0,  0, 0, 4, 4, 0, 12, 0},
 {0}
};

Fl_Menu_Item Amuse::menu_resample_choice[] = {
  {"auto", 0, 0, 0, 4, 4, 0, 12, 0},
  {"11Khz", 0, 0, 0, 4, 4, 0, 12, 0},
  {"16khz", 0, 0, 0, 4, 4, 0, 12, 0},
  {"22khz", 0, 0, 0, 4, 4, 0, 12, 0},
  {"44khz", 0, 0, 0, 4, 4, 0, 12, 0},
  {0}
};

Fl_Menu_Item Amuse::menu_channels_choice[] = {
  {"stereo", 0, 0, 0, 4, 4, 0, 12, 0},
  {"jstereo", 0, 0, 0, 4, 4, 0, 12, 0},
  {"dual chan", 0, 0, 0, 4, 4, 0, 12, 0},
  {"mono", 0, 0, 0, 4, 4, 0, 12, 0},
  {0}
};

Fl_Menu_Item Amuse::menu_filtering_choice[] = {
   {"auto", 0, 0, 0, 4, 4, 0, 12, 0},
   {"none", 0, 0, 0, 4, 4, 0, 12, 0},
   {"manual", 0, 0, 0, 4, 4, 0, 12, 0},
   {0}
};
   
Fl_Menu_Item* Amuse::eight = Amuse::menu_bitrate_choice + 0;
Fl_Menu_Item* Amuse::sixteen = Amuse::menu_bitrate_choice + 1;
Fl_Menu_Item* Amuse::twentyfour = Amuse::menu_bitrate_choice + 2;
Fl_Menu_Item* Amuse::thirtytwo = Amuse::menu_bitrate_choice + 3;
Fl_Menu_Item* Amuse::fiftysix = Amuse::menu_bitrate_choice + 4;
Fl_Menu_Item* Amuse::sixtyfour = Amuse::menu_bitrate_choice + 5;
Fl_Menu_Item* Amuse::ninetysix = Amuse::menu_bitrate_choice + 6;
Fl_Menu_Item* Amuse::onetwentyeight = Amuse::menu_bitrate_choice + 7;

Fl_Menu_Item Amuse::menu_login[] = {
 {"x_audiocast", 0,  0, 0, 4, 4, 0, 12, 0},
 {"icy", 0,  0, 0, 4, 4, 0, 12, 0},
 {0}
};
Fl_Menu_Item* Amuse::icy = Amuse::menu_login + 0;
Fl_Menu_Item* Amuse::x_audiocast = Amuse::menu_login + 1;

Amuse::Amuse() {

  synchronous.registerObjectName("Amuse");

  //  Fl_Double_Window* w;
  Fl_Window* w;
  { 
    sprintf(muse_version, "%s %s", PACKAGE, VERSION);
    //    Fl_Double_Window* o = mainwindow = new Fl_Double_Window(735, 500, muse_version);
    Fl_Window* o = mainwindow = new Fl_Double_Window(735, 500, muse_version);
    w = o;
    o->user_data((void*)(this));

    { Fl_Menu_Bar* o = main_menu = new Fl_Menu_Bar(0, 0, 191, 22, " ");
      o->box(FL_FLAT_BOX);
      o->down_box(FL_FLAT_BOX);
      o->selection_color(95);
      o->labeltype(FL_NO_LABEL);
      o->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
      o->menu(menu_main_menu);
    }
    { Fl_Output* o = status_ice = new Fl_Output(170, 10, 300, 12);
      o->box(FL_FLAT_BOX);
      o->color(49);
      o->textcolor(36);
      o->labeltype(FL_NO_LABEL);
      o->textsize(10);
      o->selection_color(49);
      o->selection_color(49);
      //o->value("Ice Area");
    }
    { Fl_Light_Button* o = talk_button = new Fl_Light_Button(550, 5, 49, 20, "talk");
      o->labeltype(FL_ENGRAVED_LABEL);
      o->labelsize(12);
      o->type(FL_TOGGLE_BUTTON);
      o->shortcut(0x20);
      o->callback(cb_set_talk);
    }
    { Fl_Light_Button* o = linein_button = new Fl_Light_Button(601, 5, 62, 20, "line-in");
      o->labeltype(FL_ENGRAVED_LABEL);
      o->labelsize(12);
      o->type(FL_TOGGLE_BUTTON);
      o->callback(cb_set_linein);
    }
    { Fl_Light_Button* o = lineout_button = new Fl_Light_Button(665, 5, 68, 20, "speaker");
      o->labeltype(FL_ENGRAVED_LABEL);
      o->labelsize(12);
      o->type(FL_TOGGLE_BUTTON);
      o->callback(cb_set_lineout);
    }
    
    { 
      Fl_Button* o = version = new Fl_Button(628, 480, 77, 20, muse_version);
      o->box(FL_FLAT_BOX);
      o->down_box(FL_FLAT_BOX);
      o->labeltype(FL_ENGRAVED_LABEL);
      o->labelfont(2);
      o->labelsize(12);
    }

    { Fl_Button* o = significalafica = new Fl_Button(705, 475, 23, 25);
      o->box(FL_FLAT_BOX);
      o->down_box(FL_FLAT_BOX);
      pixmap_peach.label(o);
      //o->callback(cb_open_info);
    }
    { Fl_Output* o = status_main = new Fl_Output(300, 485, 300, 12);
      o->box(FL_FLAT_BOX);
      o->color(49);
      o->textcolor(40);
      o->textfont(2);
      o->labeltype(FL_NO_LABEL);
      o->textsize(10);
      o->value("Status Area");
    }
    
    {
      Fl_Tabs* o = tab = new Fl_Tabs(0, 30, 735, 445);
      o->box(FL_FLAT_BOX);
      //      o->box(FL_DOWN_BOX);
      {
	Fl_Group* o = chantab = new Fl_Group(0, 30, 720, 425, " chans ");
	o->labeltype(FL_ENGRAVED_LABEL);
	o->labelsize(12);
	chan[0] = new GuiChannel(10, 50, "  ch01", 0);
	chan[1] = new GuiChannel(245, 50, "  ch02", 1);
	chan[2] = new GuiChannel(480, 50, "  ch03", 2);
	chan[3] = new GuiChannel(10, 260, "  ch04", 3);
	chan[4] = new GuiChannel(245, 260, "  ch05", 4);
	chan[5] = new GuiChannel(480, 260, "  ch06", 5);
	
	//	mix_monitor = new Level(715, 260, 10, 235); 
	
	o->end();       
      }
      
      {
	Fl_Group* o = icetab = new Fl_Group(0, 25, 720, 385, " stream ");
	o->hide();
	o->labeltype(FL_ENGRAVED_LABEL);
	o->labelsize(12);
	{
	  Fl_Input* o = ip_input = new Fl_Input(100, 100, 135, 20, "Host");
	  o->color(52);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	}

	{
	  Fl_Int_Input* o = port_input = new Fl_Int_Input(270, 100, 45, 20, "Port");
	  o->color(52);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	}
	{
	  Fl_Input* o = mount_input = new Fl_Input(370, 100, 72, 20, "MntPnt");
	  o->color(52);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	}
	{
	  Fl_Input* o = name_input = new Fl_Input(100, 130, 165, 20, "Name");
	  o->color(52);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	}
	{
	  Fl_Input* o = url_input = new Fl_Input(300, 130, 142, 20, "Url");
	  o->color(52);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	}
	{
	  Fl_Input* o = description_input = new Fl_Input(100, 160, 342, 20, "Description");
	  o->color(52);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	}
	{
	  Fl_Input* o = pass_input = new Fl_Input(505, 100, 80, 20, "Pass");
	  o->type(5);
	  o->color(52);
	  o->selection_color(52);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	}
	{
	  Fl_Choice* o = login_choice = new Fl_Choice(590, 130, 95, 20, "login type");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(11);
	  o->menu(menu_login);
	  o->value(0);
	  //o->callback(cb_set_bitrate);
	}
	{
	  Fl_Light_Button* o = connect_button = new Fl_Light_Button(590, 100, 95, 20, "connect");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->selection_color(FL_RED);
	  o->type(FL_TOGGLE_BUTTON);
	  o->callback(cb_set_icecast);
	} 
	/* SAVE FILE TO DISK PART */
	{
	  Fl_Light_Button* o = filesave_button = new Fl_Light_Button(100, 250, 100, 20, "Save to File:");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->selection_color(FL_RED);
	  o->type(FL_TOGGLE_BUTTON);
	  o->callback(cb_save_file);
	} 
	{
	  Fl_Input* o = filesave_input = new Fl_Input(205, 250, 145, 20, "Save to File:");
	  o->color(52);
	  o->labeltype(FL_NO_LABEL);
	  o->labelsize(12);
	  o->textsize(12);
	  //o->align(FL_ALIGN_TOP_LEFT);
	  //	     o->value("saved.mp3");
	}
	{ 
	  Fl_Button* o = filebrowse_button = new Fl_Button(360, 250, 72, 20, "Browse..");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->callback(cb_browse_file);
	}
	/* --- end FILE TO DISK --- */

	{
	  Fl_Choice* o = bitrate_choice = new Fl_Choice(590, 250, 95, 20, "bitrate");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(11);
	  o->menu(menu_bitrate_choice);
	  o->value(2);
	}
	
	{
	  Fl_Choice* o = resample_choice = new Fl_Choice(590, 280, 95, 20, " frequency ");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(11);
	  o->menu(menu_resample_choice);
	  o->value(0);
	}
	
	{
	  Fl_Choice* o = channels_choice = new Fl_Choice(590, 310, 95, 20, " channels ");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(11);
	  o->menu(menu_channels_choice);
	  o->value(3);
	}
	
	{
	  Fl_Value_Slider* o = quality_slide = new Fl_Value_Slider(100, 280, 130, 20, " quality (1 = best)");
	  o->type(1);
	  o->selection_color(15);
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(11);
	  o->minimum(1);
	  o->maximum(9);
	  o->step(1);
	  o->value(4);
	  //	       o->callback((Fl_Callback*)callback);
	}
	
	/* FREQUENCY FILTERING */
	{
	  Fl_Choice* o = filtering_choice = new Fl_Choice(360, 280, 72, 20, " frequency filtering ");
	  o->labeltype(FL_ENGRAVED_LABEL);
	  o->labelsize(12);
	  o->textsize(11);
	  o->menu(menu_filtering_choice);
	  o->value(0);
	  o->callback(cb_filtering_choice);
	}
	{
	  Fl_Input* o = lowpassfreq_input = new Fl_Input(300, 310, 50, 20, " lowpass Hz ");
	  o->color(52);
	  o->labelsize(9);
	  o->textsize(12);
	  o->deactivate();
	}
	
	{
	  Fl_Input* o = highpassfreq_input = new Fl_Input(410, 310, 50, 20, " highpass Hz ");
	  o->color(52);
	  o->labelsize(9);
	  o->textsize(12);
	  o->deactivate();
	}
	/* ----------------------- */ 
	{
	  Fl_Output* o = status_lame = new Fl_Output(100, 220, 585, 24);
	  o->box(FL_ENGRAVED_BOX);
	  o->color(47);
	  o->textcolor(36);
	  o->labeltype(FL_NO_LABEL);
	  o->textsize(12);
	  o->selection_color(49);
	  o->selection_color(49);
	  o->value("no encoder detected!");
	}
	
	{
	  Fl_Output* o = status_libshout = new Fl_Output(100, 70, 585, 24);
	  o->box(FL_ENGRAVED_BOX);
	  o->color(47);
	  o->textcolor(36);
	  o->labeltype(FL_NO_LABEL);
	  o->textsize(12);
	  o->selection_color(49);
	  o->selection_color(49);
	  o->value("Streaming source authentication");
	}

	o->end();
      }
      
    } /* tabs */
    
    o->end();
    o->resizable(o);
    
  }

  quit = false;

}

  

void Amuse::run() {

  mainwindow->show();
  while(!quit) {	

    if(Fl::wait(0.05)<0.0)
      quit = true;

  }

  fprintf(stderr,"Amuse gui exiting\n");

}
