/* muse.cpp - MuSE
   (c)2000 Denis Roio aka jaromil
   see COPYING for licensing stuff

   here command line is parsed
   synchronous object are launched depending on mode selection
   threads are joined at the end */

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "config.h"
#include "generic.h"
#include "jmixer.h"
#include "netlistener.h"
#include "scheduler.h"
#include "gui_muse.h"
#include "gui_callbacks.h"
//#include "libsync/synchronous.h"

/* command line stuff */

char *version =
"%s version %s [ http://muse.dyne.org ]\n"
"(c)2000-2001 jaromil & august black\n\n";

char *help =
"Usage: muse [options]\n\n"
"Generic options\n"
"-h --help\tthis help\n"
"-v --version\tversion information\n"
//"-d --debug\tturn debug informations on\n"
"-D --daemon\tdaemon mode: do not spawn amuse local interface\n"
"-l --playlist\tuse filename as playlist (default ./playlist)\n"
"-q --queue\tqueue mode: process filename as a queue (default ./queue)\n"
"\n"
"--\n"
"for any further information refer to http://muse.dyne.org\n";

static const struct option long_options[] = {
  { "help", no_argument, NULL, 'h' },
  { "version", no_argument, NULL, 'v' },
  { "debug", no_argument, NULL, 'd' },
  { "daemon", no_argument, NULL, 'D' },
  { "playlist", required_argument, NULL, 'l' },
  { "queue", required_argument, NULL, 'q' },
  { 0, 0, 0, 0 }
};

char *short_options = "hvdDl:q:";

/* misc settings */
bool debug = false;
bool daemon_mode = false;
bool queue_mode = false;
char playlist[256];
char queue_file[256];

void take_args(int argc, char **argv) {
  int res;
  char temp[256];

  strcpy(playlist,"./playlist");
  strcpy(queue_file,"./queue");

  sprintf(temp,version,PACKAGE,VERSION,PACKAGE);
  fprintf(stderr,"%s",temp);
  
  do {
    res = getopt_long(argc, argv, short_options, long_options, NULL);

    switch(res) {
    case 'h':
      fprintf(stderr,"%s",help);
      exit(0);
      break;
    case 'v':
      exit(0);
      break;
    case 'd':
      debug = true;
      break;
    case 'D':
      daemon_mode = true;
      break;
    case 'q':
      strncpy(queue_file,optarg,256);
      queue_mode = true;
    case 'l':
      strncpy(playlist,optarg,256);
      break;
    }
  } while(res > 0);

}

void create_config() {
  FILE *fd;
  char *home;
  char temp[MAX_PATH_SIZE];
  
  home = getenv("HOME");
  
  sprintf(temp,"%s/.muse/muse.conf",home);
  fd = fopen(temp,"w");
  if(fd==NULL) {
    fprintf(stderr,"i don't reach to create a new default config file!\nbailing out :(\n");
    exit(1);
  }

  fprintf(stderr,"A new default config file has been created in %s\n",temp);
  sprintf(temp,"# config file generated automaticly by MuSE\n# do not edit unless you know what you are doing\n");
  fputs(temp,fd);
  sprintf(temp,"version\t%s\n",VERSION);
  fputs(temp,fd);

  fclose(fd);

}

void check_config() {
  FILE *fd;
  char *home;
  char temp[MAX_PATH_SIZE];
  
  home = getenv("HOME");

  sprintf(temp,"%s/.muse",home);
  mkdir(temp,0744);

  sprintf(temp,"%s/.muse/muse.conf",home);
  fd = fopen(temp,"r");
  if(fd != NULL) {
    
    /* here we read what we need from muse.conf */
    fclose(fd);
  } else {
    fprintf(stderr,"This seems the first time you run MuSE\n");
    create_config();
  }  
}

int main(int argc, char **argv) {

  take_args(argc, argv);

  /* change those for having the stderr flodded by
     the libsync threaded kernel debugging messages */
  LTI::Kernel::trace = false;
  LTI::Kernel::debug = false;

  fprintf(stderr,"starting engine...\n");

  //  check_config();

  Stream_mixer *mix = new Stream_mixer();

  if(!mix->open_soundcard()) {
    fprintf(stderr,"exiting.\n");
    exit(1);
  }

  int c;

  /* load the default playlist on every channel if present */
  for(c=0;c<MAX_CHANNELS;c++)
    mix->playlist[c].load("playlist");
  
  if(queue_mode) {

    mix->start();
  
    Scheduler *queue = new Scheduler(queue_file,mix);
    
    queue->start();
    
    queue->join();
    mix->join();
    
  } else if(daemon_mode) {
    
    mix->start();

    Net_listener *net = new Net_listener(LISTENPORT,mix);
    
    net->start();
    
    net->join();
    mix->join();
  } else {
    
    Amuse *gui = new Amuse();


    init_callbacks(mix,gui->getSynchronous());

    mix->register_gui(gui);

    gui->start();
    mix->start();

    gui->join();
    mix->quit = true;
    mix->join();
  }

  fprintf(stderr,"cya!\n");

  exit(0);
}
